#include "StdAfx.h"
#include "resource.h"
#include ".\accdlg.h"

const static char HEADER_CH[]		= "`l";
const static char HEADER_AUTOJOIN[]	= "JOIN";

enum{
	COL_CH,
	COL_AUTOJOIN,
	NUM_OF_COL,
};

//******************************************************************
// 
//******************************************************************

CAccDlg::CAccDlg(void)
: CDlg(IDD_ACC_SET)
, m_bCanceled(true)
{
	m_set.port = 6667;
}

CAccDlg::~CAccDlg(void)
{
	ConChList::iterator i;

	if( m_bCanceled ){
		for( i = m_created.begin(); i != m_created.end(); ++i )
			delete *i;
	}else{
		for( i = m_deleted.begin(); i != m_deleted.end(); ++i )
			delete *i;
	}
}

//******************************************************************
// 
//******************************************************************

BOOL CAccDlg::DlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg ){
	case WM_NOTIFY:
		if( LOWORD(wParam) == IDC_CH ){
			switch( ((LPNMHDR)lParam)->code ){
			case NM_DBLCLK:
				OnListDblClick((NMITEMACTIVATE*)lParam);
				break;
			default:
				return FALSE;
			}
			return TRUE;
		}
		return FALSE;
	case WM_COMMAND:
		switch( LOWORD(wParam) ){
		case IDC_ADD: OnClickAdd(); break;
		case IDC_TOGGLE: OnClickToggle(); break;
		case IDC_DEL: OnClickDel(); break;
		case IDC_ACCOUNT:
			if( HIWORD(wParam) == EN_CHANGE )
				SetEnable(IDOK, ::GetWindowTextLength((HWND)lParam) != 0);
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

BOOL CAccDlg::InitDlgCtrl()
{
	LVCOLUMN lvc;
	HWND hItem;

	SetItemText(IDC_ACCOUNT, m_set.acc.c_str());
	SetItemText(IDC_HOST, m_set.host.c_str());
	SetItemText(IDC_PORT, m_set.port);
	SetItemText(IDC_PASS, m_set.pass.c_str());
	SetItemText(IDC_NICK, m_set.nick.c_str());

	// Xg
	hItem = GetDlgItem(IDC_CH);
	ListView_SetExtendedListViewStyle(hItem, LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_LABELTIP);

	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM | LVCF_TEXT;
	lvc.fmt = LVCFMT_LEFT;
	lvc.pszText = (LPSTR)HEADER_CH;
	lvc.cx = 200;
	lvc.iSubItem = COL_CH;
	ListView_InsertColumn(hItem, COL_CH, &lvc);

	lvc.pszText = (LPSTR)HEADER_AUTOJOIN;
	lvc.cx = 80;
	lvc.iSubItem = COL_AUTOJOIN;
	ListView_InsertColumn(hItem, COL_AUTOJOIN, &lvc);

	UpdateList();
	return TRUE;
}

BOOL CAccDlg::GetDlgCtrlValue()
{
	GetItemText(IDC_ACCOUNT, m_set.acc);
	GetItemText(IDC_HOST, m_set.host);
	m_set.port = (unsigned short)GetItemTextInt(IDC_PORT);
	GetItemText(IDC_PASS, m_set.pass);
	GetItemText(IDC_NICK, m_set.nick);

	return m_set.acc.length() > 0;
}

//******************************************************************
// 
//******************************************************************

void CAccDlg::OnListDblClick(NMITEMACTIVATE* pNMIA)
{
	ToggleAutoJoin(pNMIA->iItem);
}

void CAccDlg::OnClickAdd()
{
	CChDlg dlg;

	if( dlg.DoModal(m_hInstance, m_hDlgWnd) == IDOK ){
		int size = m_set.chs.size();
		LPCSTR name = dlg.GetCh();
		for( int i = 0; i < size; ++i ){
			if( m_set.chs[i]->name == name )
				return;
		}
		ConChannel* ch = new ConChannel;
		ch->name = name;
		ch->bAutoJoin = dlg.GetAutoJoin();
		m_set.chs.push_back(ch);
		m_created.push_back(ch);

		UpdateList();
		ListView_SetItemState(GetDlgItem(IDC_CH), m_set.chs.size() - 1, LVIS_SELECTED, LVIS_SELECTED);
		ListView_SetSelectionMark(GetDlgItem(IDC_CH), m_set.chs.size() - 1);
	}
}

void CAccDlg::OnClickToggle()
{
	int ich = GetCurSelect();

	ToggleAutoJoin(ich);
}

void CAccDlg::OnClickDel()
{
	int ich = GetCurSelect();

	if( (0 <= ich) && ((u_int)ich < m_set.chs.size()) ){
//		delete m_set.chs[ich];
		m_deleted.push_back(m_set.chs[ich]);
		m_set.chs.erase(m_set.chs.begin() + ich);

		UpdateList();
		ListView_SetItemState(GetDlgItem(IDC_CH), ich - 1, LVIS_SELECTED, LVIS_SELECTED);
		ListView_SetSelectionMark(GetDlgItem(IDC_CH), ich - 1);
	}
}

//******************************************************************
// 
//******************************************************************

int CAccDlg::GetCurSelect()
{
	HWND hList = GetDlgItem(IDC_CH);
	return ListView_GetSelectionMark(hList);
}

void CAccDlg::ToggleAutoJoin(int ich)
{
	if( (0 <= ich) && ((u_int)ich < m_set.chs.size()) ){
		HWND hList = GetDlgItem(IDC_CH);
		ConChannel *ch = m_set.chs[ich];
		ch->bAutoJoin = ! ch->bAutoJoin;

		UpdateList();
		ListView_SetItemState(hList, ich, LVIS_SELECTED, LVIS_SELECTED);
		ListView_SetSelectionMark(hList, ich);
	}
}

void CAccDlg::UpdateList()
{
	int iItem;
	HWND hList;
	LVITEM lvi;
	int ed;

	hList = GetDlgItem(IDC_CH);
	::SendMessage(hList, LVM_DELETEALLITEMS, 0, 0);

	iItem = 0;
	lvi.mask = LVIF_TEXT;
	ed = m_set.chs.size();
	for( int i = 0; i < ed; ++i ){
		const ConChannel * ch = m_set.chs[i];

		lvi.iItem = iItem;
		lvi.iSubItem = COL_CH;
		lvi.pszText = (LPSTR)ch->name.c_str();
		ListView_InsertItem(hList, &lvi);

		ListView_SetItemText(hList, iItem, COL_AUTOJOIN,
			(LPSTR)(ch->bAutoJoin ? "": "~"));
		iItem++;
	}

	for( int i = 0; i < NUM_OF_COL; ++i ){
		ListView_SetColumnWidth(hList, i, LVSCW_AUTOSIZE_USEHEADER);
	}
}
