#include "StdAfx.h"
#include <stdarg.h>
#include ".\irccmd.h"
#include "../nsmsgs/general_strings.h"

struct CmdPenalty{
	LPCSTR cmd;
	int penalty;
	bool operator <(LPCSTR cmp) const
	{ return strcmp(cmd, cmp) < 0; }
	bool operator <(const CmdPenalty& cp) const
	{ return strcmp(cmd, cp.cmd) < 0; }
};

bool operator <(LPCSTR cmp, const CmdPenalty& cp)
{
	return strcmp(cmp, cp.cmd) < 0;
}


CIRCCmd::CIRCCmd(void)
: m_buf_filled(0)
{
}

CIRCCmd::~CIRCCmd(void)
{
}

bool CIRCCmd::Cmd(LPCSTR msg)
{
	char buf[IRC_MESSAGE_SIZE];
	bool ret;
	LPCSTR from, host, param;
	LPSTR cmd;
	int penalty;

	SplitMsg(msg, buf, from, host, cmd, param);
	_strupr_s(cmd, IRC_MESSAGE_SIZE);
	penalty = GetCmdPenalty(cmd);

	Clear();
	ret = Append(cmd, strlen(cmd));
	ret = ret && Append(CRLF_STRING, LEN_OF(CRLF_STRING));
	if( ret ) Send(penalty);

	return ret;
}
/*
bool CIRCCmd::Cmd(LPCSTR cmd, ...)
{
	char buf[IRC_MESSAGE_SIZE];
	va_list args;
	LPCSTR arg;
	int penalty;

	strcpy_s(buf, cmd);
	_strupr_s(buf);
	penalty = GetCmdPenalty(buf);

	Clear();
	if( ! Append(buf, strlen(buf)) )
		return false;

	va_start(args, cmd);
	do{
		arg = va_arg(args, LPCSTR);
		if( ! Append(arg, strlen(arg)) ){
			va_end(args);
			return false;
		}
	}
	while( arg != CRLF_STRING );
	va_end(args);

	Send(penalty);
	return true;
}
*/
void CIRCCmd::Pass(LPCSTR pass)
{
	ConcatSend(2, "PASS :", pass, CRLF_STRING);
}

void CIRCCmd::User()
{
	ConcatSend(2, "USER x 8 x x", CRLF_STRING);
}

void CIRCCmd::Ison()
{
}

void CIRCCmd::Whois()
{
}

void CIRCCmd::List()
{
}

void CIRCCmd::Who(LPCSTR q)
{
	ConcatSend(2, "WHO ", q, CRLF_STRING);
}

void CIRCCmd::Join(LPCSTR ch)
{
	ConcatSend(3, "JOIN ", ch, CRLF_STRING);
}

void CIRCCmd::Part(LPCSTR ch)
{
	ConcatSend(5, "PART ", ch, CRLF_STRING);
}

void CIRCCmd::Topic()
{
}

void CIRCCmd::Invite(LPCSTR user, LPCSTR ch)
{
	ConcatSend(3, "INVITE ", user, SPACE_DELIMITER, ch, CRLF_STRING);
}

void CIRCCmd::Mode()
{
}

void CIRCCmd::Names()
{
}

bool CIRCCmd::Privmsg(LPCSTR target, LPCSTR msg)
{
	LPCSTR msg_st, msg_ed;
	const int tlen = strlen(target);
	int mlen = strlen(msg);
	bool bSplit = false;

	msg_st = msg;
	while( mlen > 400 ){
		msg_ed = (LPCSTR)_mbsninc((const u_char*)msg_st, 200);

		Clear();
		Append("PRIVMSG ", 8);
		Append(target, tlen);
		Append(" :", 2);
		Append(msg_st, msg_ed - msg_st);
		Append(CRLF_STRING, LEN_OF(CRLF_STRING));
		Send(2);

		mlen -= msg_ed - msg_st;
		msg_st = msg_ed;
		bSplit = true;
	}
	ConcatSend(2, "PRIVMSG ", target, " :", msg_st, CRLF_STRING);
	return ! bSplit;
}

void CIRCCmd::Nick(LPCSTR nick)
{
	ConcatSend(4, "NICK ", nick, CRLF_STRING);
}

void CIRCCmd::Away()
{
}

void CIRCCmd::Kick(LPCSTR ch, LPCSTR user)
{
	ConcatSend(5, "KICK ", ch, SPACE_DELIMITER, user, CRLF_STRING);
}

void CIRCCmd::Pong(LPCSTR target)
{
	ConcatSend(2, "PONG ", target, CRLF_STRING);
}

void CIRCCmd::Quit(LPCSTR msg)
{
	ConcatSend(-10, "QUIT :", msg, CRLF_STRING);
}

bool CIRCCmd::ConcatSend(int penalty, ...)
{
	va_list args;
	LPCSTR arg;

	Clear();

	va_start(args, penalty);
	do{
		arg = va_arg(args, LPCSTR);
		if( ! Append(arg, strlen(arg)) ){
			va_end(args);
			return false;
		}
	}
	while( arg != CRLF_STRING );
	va_end(args);

	Send(penalty);
	return true;
}

void CIRCCmd::SplitMsg(LPCSTR in, LPSTR buf, LPCSTR &from,
					   LPCSTR &host, LPSTR &cmd, LPCSTR &param)
{
	LPCSTR cmd_st, cmd_ed, from_ed;
	LPSTR buf_pos;
	LPSTR cmd_buf;
	int len;

	// from, host 擾
	buf_pos = buf;
	if( *in == ':' ){
		cmd_st = strchr(in, ' ');
		if( cmd_st == NULL ) return;
		cmd_st++;

		from_ed = strchr(in+1, '!');
		if( (from_ed != NULL) && (from_ed < cmd_st) ){
			from = buf_pos;
			len = from_ed - (in + 1);
			memcpy(buf_pos, in+1, len);
			buf_pos[len] = '\0';
			buf_pos += len + 1;
		}else{
			from_ed = in;
			from = NULL_STRING;
		}

		host = buf_pos;
		len = cmd_st - from_ed - 2;
		memcpy(buf_pos, from_ed+1, len);
		buf_pos[len] = '\0';
		buf_pos += len + 1;
	}else{
		from = NULL_STRING;
		host = NULL_STRING;
		cmd_st = in;
	}
/*
                   from_ed       cmd_st
                   |             |
	in	= :fromname!abc@hostname command parameter...
	buf	= fromname\0hostname\0  
	      |         |         |
		  from      host      buf_pos

*/
	cmd_buf = buf_pos;
	cmd = cmd_buf;

	// cmd 擾
	cmd_ed = strchr(cmd_st, ' ');
	if( cmd_ed == NULL ){
		cmd_ed = cmd_st + strlen(cmd_st);
		if( cmd_ed <= cmd_st ) return;

		param = cmd_ed;
	}else{
		param = cmd_ed + 1;
	}
	len = cmd_ed - cmd_st;
	memcpy(cmd_buf, cmd_st, len);
	cmd_buf[len] = '\0';
}

// (1).p[^14ȉōŌ̃p[^RŎn܂
// (2).p[^15
// (3).p[^14ȉōŌ̃p[^RȊOŎn܂
void CIRCCmd::SplitParam(ParamList &list, LPSTR param)
{
	LPSTR pos, p_st;

	p_st = param;
	for( int i = 0; i < 14 /* (2) */ ; ++i ){
		if( *p_st == ':') break;	// (1)
		pos = strchr(p_st, ' ');
		if( pos == NULL ){ // (3)
			list.push_back(p_st);
			return;
		}
		*pos = '\0';
		list.push_back(p_st);
		p_st = pos + 1;
	}

	// (1)(2)
	if( *p_st == ':' ) p_st++;
	list.push_back(p_st);
}

#define SIZE_OF_ARRAY(ary) \
	(sizeof(ary)/sizeof(*(ary)))
int CIRCCmd::GetCmdPenalty(LPCSTR cmd)
{
	static const CmdPenalty penaltyTable[] = {
		{"AWAY", 3},
		{"INFO", 6},
		{"INVITE", 3},
		{"ISON", 2},
		{"JOIN", 3},
		{"KICK", 5},
		{"LINKS", 3},
		{"LIST", 3},
		{"LUSERS", 3},
		{"MODE", 3},
		{"MOTD", 3},
		{"NAMES", 3},
		{"NICK", 4},
		{"NOTICE", 2},
		{"OPER", 4},
		{"PART", 5},
		{"PING", 2},
		{"PONG", 2},
		{"PRIVMSG", 2},
		{"STATS", 3},
		{"TIME", 3},
		{"TOPIC", 4},
		{"UMODE", 2},
		{"USER", 3},
		{"USERHOST", 2},
		{"USERS", 3},
		{"VERSION", 3},
		{"WHO", 6},
		{"WHOIS", 2},
		{"WHOWAS", 3},
	};
	const CmdPenalty* pcp;

	pcp = std::lower_bound(&penaltyTable[0], 
		&penaltyTable[0] + SIZE_OF_ARRAY(penaltyTable), cmd);

	return (strcmp(cmd, pcp->cmd) == 0) ? pcp->penalty: 3;
}
