#include "StdAfx.h"
#include <sstream>
#include ".\ircconnection.h"
#include "IRCSettings.h"
#include "IRCSession.h"
#include "common.h"
#include "../nsmsgs/cs_conv.h"
#include "../nsmsgs/DynamicArray.h"
#include "../nsmsgs/general_strings.h"
#include "../nsmsgs/debug.h"

#define DEFINE_FUNC_SWITCH_STRUCT(name) \
	{#name, CIRCConnection::Process##name},
#define DEFINE_FUNC_SWITCH_FUNC(name, from, host, param) \
	void CIRCConnection::Do##name(LPCSTR from, LPCSTR host, ParamList &param)
#define EXEC_IRC_SES(ch, func) \
	{ \
		CIRCSession *s = GetSession(ch); \
		if( s != NULL ) s->func; \
	}
		
//******************************************************************
// 
//******************************************************************

CIRCConnection::CIRCConnection(PLoginInfo pli, HNsmConnection &hCon)
: m_nsm(pli->lpAccount, hCon)
, m_pSocket(NULL)
, m_bConnected(false)
, m_acc(pli->lpAccount)
{
	AccSetting set;

	m_lock = ::CreateSemaphore(NULL, 1, 1, NULL);
	set.acc = pli->lpAccount;

	if( IRC_SETTINGS.GetAccSetting(set) ){
		Lock();
		m_pSocket = new CIRCSocket(set.host.c_str(), set.port, *this);
		m_irc.SetSocket(m_pSocket);
		m_chs = set.chs;

		Login(set, NULL);//pli->lpPassword);
		Unlock();
	}else{
		DoError("AJEgݒł");
	}
}

CIRCConnection::~CIRCConnection(void)
{
	delete m_pSocket;

	SesMap::iterator i;
	for( i = m_slist.begin(); i != m_slist.end(); ++i ){
		delete i->second;
	}

	int size = m_chs.size();
	for( int i = 0; i < size; ++i )
		delete m_chs[i];

	::CloseHandle(m_lock);
}

//******************************************************************
// CIRCPluginĂяo
//******************************************************************

void CIRCConnection::ChangeUserName(LPCWSTR nick)
{
	std::string str;
	std::string snick;

	UnicodeToSJIS(nick, snick);

	Lock();
	m_irc.Nick(snick.c_str());
	Unlock();
}

void CIRCConnection::RemMember(LPCSTR acc)
{
	const int bufSize = strlen(acc)+1;
	DynamicArray<char> buf(bufSize);
	LPSTR pos;

	strcpy_s(buf, bufSize, acc);
	pos = strchr(buf, ' ');
	if( pos == NULL ) return;
	*pos++ = '\0';

	Lock();
	m_irc.Kick(buf, pos);
	Unlock();
}

void CIRCConnection::AddGroup(LPCWSTR ch)
{
	int gid;
	ConChannel *cc;
	std::string sch;

	UnicodeToSJIS(ch, sch);

	Lock();
	gid = m_nsm.GetGroupId(sch.c_str());
	if( gid == 0 ){
		cc = new ConChannel();
		cc->bAutoJoin = false;
		std::swap(sch, cc->name);
        
		m_chs.push_back(cc);
		m_nsm.AddChannel(cc->name.c_str());
	}

	SaveConChList();
	Unlock();
}

void CIRCConnection::RemGroup(int nGroupID)
{
	ConChList::iterator i;
	std::string name;

	Lock();
	m_nsm.GetGroupName(nGroupID, name);
	m_nsm.RemChannel(nGroupID);

	for( i = m_chs.begin(); i != m_chs.end(); ++i ){
		if( (*i)->name == name ){
			delete *i;
			m_chs.erase(i);
			break;
		}
	}

	SaveConChList();
	Unlock();
}

HNsmSession CIRCConnection::SesOpen(LPCSTR acc)
{
	LPCSTR name;
	CIRCSession *s;
	HNsmSession hSes;

    name = strchr(acc, ' ');
	name = (name == NULL) ? acc: name + 1;

	Lock();
	if( GetSession(acc) == NULL ){
		s = CreateSession(name);

		if( IsChannel(s->GetName()) ){
			m_irc.Join(name);
		}else{
			s->AddMember(name, name);
			s->Connected();
		}

		hSes = s->GetNsmHandle();
	}else{
		hSes = NULL;
	}
	Unlock();

	return hSes;
}

void CIRCConnection::SesClose(HNsmSession ses)
{
	Lock();
	const CIRCSession* s = GetSession(ses);

	if( s != NULL ){
		if( m_bConnected && IsChannel(s->GetName()) ){
			m_irc.Part(s->GetName());
		}else{
			DeleteSession(s->GetName());
		}
	}
	Unlock();
}

void CIRCConnection::SesInvite(HNsmSession ses, LPCSTR acc)
{
	LPCSTR pos, nick;
	std::string str;

	Lock();
	const CIRCSession* s = GetSession(ses);

	if( s == NULL ){
		_ASSERT(FALSE);
		Unlock();
		return;
	}

	if( ! IsChannel(s->GetName()) ){
		m_nsm.Warning("11̉bɏ҂͂ł܂");
		Unlock();
		return;
	}

	pos = strchr(acc, ' ');
	nick = (pos == NULL) ? acc: pos + 1;

	m_irc.Invite(nick, s->GetName());
	Unlock();
}

void CIRCConnection::SesSendMsg(HNsmSession ses, LPCWSTR msg)
{
	std::string str;
	std::string sjis;
	LPCSTR msg_st, msg_ed;

	Lock();

	// MZbV擾
	const CIRCSession* s = GetSession(ses);
	if( s == NULL ){
		_ASSERT(FALSE);
		Unlock();
		return;
	}

	// bZ[WSJISɕϊ
	UnicodeToSJIS(msg, sjis);
	if( sjis.length() == 0 ){
		Unlock();
		return;
	}

	// 1sƂSendMsg֓n
	msg_st = sjis.c_str();
	for( msg_ed = msg_st; *msg_ed; ++msg_ed ){
		if( (*msg_ed == '\r') || (*msg_ed == '\n') ){
			if( msg_ed > msg_st ){
				str.assign(msg_st, msg_ed - msg_st);
				SendMsg(s, str);
			}
			msg_st = msg_ed + 1;
		}
	}
	if( *msg_st != '0' ){
		str.assign(msg_st);
		SendMsg(s, str);
	}
	Unlock();
}


//******************************************************************
// 
//******************************************************************

void CIRCConnection::Login(AccSetting &set, LPCSTR pass)
{
	if( pass != NULL )
		m_irc.Pass(pass);

	m_irc.Nick(set.nick.c_str());
	m_irc.User();
}

void CIRCConnection::SendMsg(const CIRCSession *s, std::string &str)
{
	bool ret;

	if( str[0] == '/' ){
		ret = m_irc.Cmd(str.c_str()+1);
		if( ! ret )
			m_nsm.Warning("R}hߑM܂ł");
	}else{
		ret = m_irc.Privmsg(s->GetName(), str.c_str());
		if( ! ret )
			m_nsm.Warning("bZ[WߕsɕđM܂");
	}
}

//******************************************************************
// 
//******************************************************************

//******************************************************************
// CIRCSocketĂяo
//******************************************************************

// lineɂIRCbZ[W1sn
// \r\n͂ĂȂ
void CIRCConnection::DoExec(LPCSTR line)
{
	LPCSTR from, host, param;
	LPSTR cmd;
	char buf[IRC_MESSAGE_SIZE];
	char buf2[IRC_MESSAGE_SIZE];
	int numeric;

	CIRCCmd::SplitMsg(line, buf, from, host, cmd, param);

DEBUG_CODE(
_RPT4(_CRT_WARN, "from='%s' host='%s' cmd='%s' param='%s'\n",
	  from, host, cmd, param);
);

	_strupr_s(cmd, IRC_MESSAGE_SIZE);
	FuncSwitchFunc f = FSwitch(cmd);

	strcpy_s(buf2, param);

	Lock();
	if( f != NULL ){
		ParamList p;
		CIRCCmd::SplitParam(p, buf2);
DEBUG_CODE(
for( u_int i = 0; i < p.size(); ++i ){
	_RPT2(_CRT_WARN, "param%02d='%s'\n", i, p[i]);
}
);
		f(this, from, host, p);
	}else{
		numeric = atoi(cmd);
		if( numeric >= 400 ){
			m_nsm.Warning(param);
			Console(param);
		}else if( numeric >= 0 ){
			Console(param);
		}else{
			Console(line);
		}
	}
	Unlock();
}

void CIRCConnection::DoLog(LPCSTR header, LPCSTR body) const
{
	m_nsm.DoLog(header, body);
}

void CIRCConnection::DoError(LPCSTR line)
{
	Lock();
	m_nsm.Error(line);
	Disconnected();
	Unlock();
}

//******************************************************************
// 
//******************************************************************

// welcome
DEFINE_FUNC_SWITCH_FUNC(001, /*from*/, /*host*/, param)
{
	if( param.size() < 1 ) return;

	ConChannel *cc;
	int size = m_chs.size();

	for( int i = 0; i < size; ++i ){
		cc = m_chs[i];
		m_nsm.AddChannel(cc->name.c_str());
		if( cc->bAutoJoin ){
			CreateSession(cc->name.c_str());
			m_irc.Join(cc->name.c_str());
		}
	}

	ChangeUserName(param[0]);
	Connected();
}

// away
// "<nick> :<away message>" 
DEFINE_FUNC_SWITCH_FUNC(301, /*from*/, /*host*/, param)
{
	if( param.size() < 3 ) return;

	std::string str;

	str = param[1];
	str += "AWAYԂɐݒ肳Ă܂B";
	str += CRLF_STRING;
	str += "bZ[WF";
	str += param[2];
	Console(str.c_str());
}

// unaway
DEFINE_FUNC_SWITCH_FUNC(305, /*from*/, /*host*/, /*param*/)
{
	Console("AWAYԂ͉܂");
}

// now away
DEFINE_FUNC_SWITCH_FUNC(306, /*from*/, /*host*/, /*param*/)
{
	Console("AWAYԂɐݒ肳܂");
}

// end of who
// 315 <me> <name> :End of WHO list
//      0    1      2-
DEFINE_FUNC_SWITCH_FUNC(315, /*from*/, /*host*/, param)
{
	if( param.size() < 2 ) return;

	SessionConnected(param[1]);
}

// channel mode is
// 	"<channel> <mode> <mode params>" 
DEFINE_FUNC_SWITCH_FUNC(324, /*from*/, /*host*/, param)
{
	if( param.size() < 3 ) return;
}

// no topic
DEFINE_FUNC_SWITCH_FUNC(331, /*from*/, /*host*/, param)
{
	if( param.size() < 2 ) return;

	SessionTopic(param[1], "");
}

// topic
DEFINE_FUNC_SWITCH_FUNC(332, /*from*/, /*host*/, param)
{
	if( param.size() < 3 ) return;

	SessionTopic(param[1], param[2]);
}
// inviting
// "<channel> <nick>"
DEFINE_FUNC_SWITCH_FUNC(341, /*from*/, /*host*/, param)
{
	if( param.size() < 2 ) return;

	std::string str;

	str = param[1];
	str += "";
	str += param[0];
	str += "֏҂܂B";
	Console(str.c_str());
}
// who
// 352 <me> <ch> <user> <host> <server> <nick> ("H"/"G")["*"][("@"/"+")] :<hopcount> <real name>
//      0    1    2      3      4        5       6                        7-
DEFINE_FUNC_SWITCH_FUNC(352, /*from*/, /*host*/, param)
{
	std::string acc;
	u_int mode = 0;

	if( param.size() < 7 ) return;

	if( IsMe(param[5]) ) return;
	CIRCSession *s = GetSession(param[1]);
	if( s == NULL ) return;
	if( s->IsConnected() ) return;

	acc = param[2];
	acc += "@";
	acc += param[3];

	if( strchr(param[6], '*') != NULL )
		mode |= CIRCUser::MODE_OPER;
	if( strchr(param[6], '@') != NULL )
		mode |= CIRCUser::MODE_CHOP;
	else if( strchr(param[6], '+') != NULL )
		mode |= CIRCUser::MODE_VOICE;

	s->AddMember(param[5], acc.c_str(), mode);

	m_nsm.AddMember(param[1], param[5]);
}

/*
// names
DEFINE_FUNC_SWITCH_FUNC(353, from, host, param)
{
	std::string user;
	LPCSTR user_st, user_ed;

	if( param.size() < 4 ) return;

	user_st = param[3];
	for(;;){
		if( (*user_st == '@') || (*user_st == '+') ) user_st++;
		user_ed = strchr(user_st, ' ');
		if( user_ed == NULL ) break;
		user.assign(user_st, user_ed - user_st);
		SessionMemberAdd(param[2], user.c_str());

		user_st = user_ed + 1;
	}
	user.assign(user_st);
	if( user.length() > 0 )
		SessionMemberAdd(param[2], user.c_str());
}

// end of names
DEFINE_FUNC_SWITCH_FUNC(366, from, host, param)
{
	if( param.size() < 2 ) return;

	SessionConnected(param[1]);
}
*/

// INVITE <nick> <channel>
//         0       1
DEFINE_FUNC_SWITCH_FUNC(INVITE, from, /*host*/, param)
{
	if( param.size() < 2 ) return;

	std::string str;

	str = from;
	str += "";
	str += param[1];
	str += "֏҂܂B";
	Console(str.c_str());
}

// JOIN <chs> [<keys>]
DEFINE_FUNC_SWITCH_FUNC(JOIN, from, host, param)
{
	if( param.size() < 1 ) return;

	if( IsMe(from) ){
		CIRCSession *s = GetSession(param[0]);
		if( s == NULL ) CreateSession(param[0]);

		m_irc.Who(param[0]);
	}else{
		SessionMemberAdd(param[0], from, host);
	}
}

// KICK <channel> *(","<channel>) <user> *(","<user>) [<comment>]
//       0                         1                    2
DEFINE_FUNC_SWITCH_FUNC(KICK, from, /*host*/, param)
{
	std::string msg;

	if( param.size() < 2 ) return;

	msg = from;
	msg += " KICK ";
	msg += param[1];
	msg += " from ";
	msg += param[0];
	if( param.size() > 2 ){
		msg += ": ";
		msg += param[2];
	}
	Console(msg.c_str());

	if( IsMe(param[1]) ){
		DeleteSession(param[0]);
	}else{
//		RecvMessage(from, host, param[0], msg);
		SessionMemberRemove(param[0], param[1]);
	}
}

// KILL <nickname> <comment>
//       0           1
DEFINE_FUNC_SWITCH_FUNC(KILL, from, /*host*/, param)
{
	std::string msg;

	if( param.size() < 2 ) return;

	msg = from;
	msg += " KILL ";
	msg += param[0];
	msg += ": ";
	msg += param[1];
	Console(msg.c_str());

	if( IsMe(param[0]) ){
		SesMap::iterator i;
		for( i = m_slist.begin(); i != m_slist.end(); ++i )
			i->second->Disconnect();
		Disconnected();
	}else{
		SesMap::iterator si;

		for( si = m_slist.begin(); si != m_slist.end(); ++si ){
			si->second->RemMember(from);
		}
	}
}

// MODE <target> *(("-"/"+")*<mode> *<param>)
//       0        1-
DEFINE_FUNC_SWITCH_FUNC(MODE, /*from*/, /*host*/, param)
{
	u_int mode_idx, param_idx;
	bool bAdd = true;

	if( param.size() < 2 ) return;

	if( ! IsChannel(param[0]) ) return;

	param_idx = 0;
	for( mode_idx = 1; mode_idx < param.size(); ++mode_idx ){
		if( (*param[mode_idx] == '+') || (*param[mode_idx] == '-') ){
			for( LPCSTR c = param[mode_idx]; *c; ++c ){
				switch( *c ){
				case '+':
					bAdd = true;
					break;
				case '-':
					bAdd = false;
					break;
				case 'o':
					if( param_idx <= mode_idx ) param_idx = mode_idx;
					for( param_idx++; param_idx < param.size(); ++param_idx ){
						if( *param[param_idx] != '+' && *param[param_idx] != '-' ){
							if( ! IsMe(param[param_idx]) )
								SessionMemberMode(param[0], param[param_idx], bAdd ? CIRCUser::MODE_CHOP: 0);
							break;
						}
					}
					break;

				case 'O':
				case 'v':
				case 'k':
				case 'l':
				case 'b':
				case 'e':
				case 'I':
					if( param_idx <= mode_idx ) param_idx = mode_idx;
					for( param_idx++; param_idx < param.size(); ++param_idx ){
						if( *param[param_idx] != '+' && *param[param_idx] != '-' ){
							break;
						}
					}
					break;

				case 'a':
				case 'i':
				case 'm':
				case 'n':
				case 'q':
				case 'p':
				case 's':
				case 'r':
				case 't':
				default:
					break;
				}
			}
		}

	}
}

DEFINE_FUNC_SWITCH_FUNC(NICK, from, /*host*/, param)
{
	if( param.size() < 1 ) return;

	if( IsMe(from) ){
		ChangeUserName(param[0]);
	}else{
		SesMap::iterator si;

		for( si = m_slist.begin(); si != m_slist.end(); ++si ){
			si->second->SetMemberName(from, param[0]);
		}
	}
}

DEFINE_FUNC_SWITCH_FUNC(NOTICE, from, host, param)
{
	if( param.size() < 2 ) return;

	RecvMessage(from, host, param[0], param[1]);
}

DEFINE_FUNC_SWITCH_FUNC(PART, from, /*host*/, param)
{
	if( param.size() < 1 ) return;

	if( IsMe(from) ){
		DeleteSession(param[0]);
	}else{
		SessionMemberRemove(param[0], from);
	}
}

DEFINE_FUNC_SWITCH_FUNC(PING, /*from*/, /*host*/, param)
{
	if( param.size() < 1 ) return;

	m_irc.Pong(param[0]);
}

DEFINE_FUNC_SWITCH_FUNC(PRIVMSG, from, host, param)
{
	if( param.size() < 2 ) return;

	RecvMessage(from, host, param[0], param[1]);
}

DEFINE_FUNC_SWITCH_FUNC(QUIT, from, /*host*/, /*param*/)
{
//	if( param.size() < 1 ) return;

	if( *from == '\0' ){
		SesMap::iterator i;
		for( i = m_slist.begin(); i != m_slist.end(); ++i )
			i->second->Disconnect();
		Disconnected();
	}else{
		SesMap::iterator si;

		for( si = m_slist.begin(); si != m_slist.end(); ++si ){
			si->second->RemMember(from);
		}
	}
}

DEFINE_FUNC_SWITCH_FUNC(TOPIC, /*from*/, /*host*/, param)
{
	if( param.size() < 2 ) return;

	SessionTopic(param[0], param[1]);
}


//******************************************************************
// 
//******************************************************************

#define SIZE_OF_ARRAY(ary) \
	(sizeof(ary)/sizeof(*(ary)))
CIRCConnection::FuncSwitchFunc CIRCConnection::FSwitch(LPCSTR cmd)
{
	// łȂ΂ȂȂ
	const static FuncSwitchElem funcSwitch[] = {
		DEFINE_FUNC_SWITCH_STRUCT(001) // welcome
		DEFINE_FUNC_SWITCH_STRUCT(301) // away
		DEFINE_FUNC_SWITCH_STRUCT(305) // unaway
		DEFINE_FUNC_SWITCH_STRUCT(306) // now away
		DEFINE_FUNC_SWITCH_STRUCT(315) // now away
		DEFINE_FUNC_SWITCH_STRUCT(324) // channel mode is
		DEFINE_FUNC_SWITCH_STRUCT(331) // no topic
		DEFINE_FUNC_SWITCH_STRUCT(332) // topic
		DEFINE_FUNC_SWITCH_STRUCT(341) // inviting
		DEFINE_FUNC_SWITCH_STRUCT(352) // who
//		DEFINE_FUNC_SWITCH_STRUCT(353) // names
//		DEFINE_FUNC_SWITCH_STRUCT(366) // end of names
		DEFINE_FUNC_SWITCH_STRUCT(INVITE)
		DEFINE_FUNC_SWITCH_STRUCT(JOIN)
		DEFINE_FUNC_SWITCH_STRUCT(KICK)
		DEFINE_FUNC_SWITCH_STRUCT(KILL)
		DEFINE_FUNC_SWITCH_STRUCT(MODE)
		DEFINE_FUNC_SWITCH_STRUCT(NICK)
		DEFINE_FUNC_SWITCH_STRUCT(NOTICE)
		DEFINE_FUNC_SWITCH_STRUCT(PART)
		DEFINE_FUNC_SWITCH_STRUCT(PING)
		DEFINE_FUNC_SWITCH_STRUCT(PRIVMSG)
		DEFINE_FUNC_SWITCH_STRUCT(QUIT)
		DEFINE_FUNC_SWITCH_STRUCT(TOPIC)
	};

	const FuncSwitchElem *pos;

	pos = std::lower_bound(funcSwitch, 
		funcSwitch + SIZE_OF_ARRAY(funcSwitch), cmd);

	return (strcmp(cmd, pos->cmd) == 0) ? pos->func: NULL;
}

void CIRCConnection::SaveConChList() const
{
	IRC_SETTINGS.SetConChList(m_acc.c_str(), m_chs);
}

//******************************************************************
// 
//******************************************************************

void CIRCConnection::Connected()
{
	m_nsm.Connected();
	m_bConnected = true;
}

void CIRCConnection::Disconnected()
{
	m_nsm.Disconnect();
	m_bConnected = false;
}

void CIRCConnection::ChangeUserName(LPCSTR nick)
{
	m_nsm.ChangeUserName(nick);
	m_me = nick;
}

CIRCSession* CIRCConnection::GetSession(HNsmSession hSession)
{
	std::string name;
	std::string::size_type pos;
	m_nsm.GetSessionCaption(hSession, name);
	pos = name.find(':');
	if( (pos != std::string::npos) && (pos > 0) ){
		name.resize(pos);
	}
	return GetSession(name.c_str());
}

CIRCSession* CIRCConnection::GetSession(LPCSTR name)
{
	SesMap::iterator i = m_slist.find(name);
	return (i != m_slist.end()) ? i->second: NULL;
}

CIRCSession* CIRCConnection::CreateSession(LPCSTR ch)
{
	_ASSERT( m_slist.find(ch) == m_slist.end() );
	CIRCSession *s;

	s = m_nsm.CreateSession(ch);
	m_slist[ch] = s;

	return s;
}

bool CIRCConnection::DeleteSession(LPCSTR ch)
{
	int gid;
	SesMap::iterator i;

	i = m_slist.find(ch);
	if( i == m_slist.end() ) return false;

	gid = m_nsm.GetGroupId(ch);
	if( gid > 0 ){
		m_nsm.ResetChannel(gid);
	}

	delete i->second;
	m_slist.erase(i);

	return true;
}

void CIRCConnection::SessionConnected(LPCSTR ch)
{
	EXEC_IRC_SES(ch, Connected());
}

void CIRCConnection::SessionMemberAdd(LPCSTR ch, LPCSTR nick, LPCSTR acc)
{
	if( IsMe(nick) ) return;
	EXEC_IRC_SES(ch, AddMember(nick, acc));

	m_nsm.AddMember(ch, nick);
}

void CIRCConnection::SessionMemberRemove(LPCSTR ch, LPCSTR member)
{
	EXEC_IRC_SES(ch, RemMember(member));

	m_nsm.RemMember(ch, member);
}

void CIRCConnection::SessionMemberName(LPCSTR oldName, LPCSTR newName)
{
	SesMap::iterator si;

	for( si = m_slist.begin(); si != m_slist.end(); ++si ){
		si->second->SetMemberName(oldName, newName);
	}
}

void CIRCConnection::SessionMemberMode(LPCSTR ch, LPCSTR nick, u_int mode)
{
	EXEC_IRC_SES(ch, SetMemberMode(nick, mode));
}

void CIRCConnection::SessionTopic(LPCSTR ch, LPCSTR topic)
{
	EXEC_IRC_SES(ch, SetTopic(topic));
}

void CIRCConnection::RecvMessage(LPCSTR from, LPCSTR host, LPCSTR to, LPCSTR msg)
{
	LPCSTR sname;
	CIRCSession *s;

	sname = IsMe(to) ? from: to;

	s = GetSession(sname);
	if( s == NULL ){
		s = CreateSession(sname);
		s->AddMember(from, host);
		s->Connected();
	}

	s->RecvMessage(from, msg);
}

void CIRCConnection::Console(LPCSTR line)
{
	EXEC_IRC_SES(CONSOLE_SESSION_CAPTION, 
		RecvMessage(CONSOLE_SESSION_CAPTION, line));
}
