#include "StdAfx.h"
#include <sstream>
#include ".\ircnsm.h"
#include "common.h"
#include "../nsmsgs/cs_conv.h"
#include "../nsmsgs/CommonService.h"
#include "../nsmsgs/general_strings.h"

HNsmEvent CIRCNsm::ms_hLogEvt = NULL;

//******************************************************************
// 
//******************************************************************

CIRCNsm::CIRCNsm(LPCSTR lpAcc, HNsmConnection &hCon)
: m_hCon( NSM_SERVICE.CreateConnection(THIS_PROTOCOL, lpAcc) )
, m_gid(1)
{
	hCon = m_hCon;
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_STATUS, NMCS_CONNECTING);
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_STATUS, NMST_ONLINE);
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_USER_ACCOUNT, lpAcc);
}

CIRCNsm::~CIRCNsm(void)
{
	NSM_SERVICE.DeleteConnection(m_hCon);
}

//******************************************************************
// 
//******************************************************************

void CIRCNsm::Connected() const
{
	NSM_SERVICE.CreateGroup(m_hCon, -1, L"R\[");
	NSM_SERVICE.CreateConnectionMember(m_hCon, -1, 
		CONSOLE_SESSION_CAPTION, L"R\[J", NMST_ONLINE);
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_STATUS, NMCS_CONNECTED);
}

void CIRCNsm::Disconnect() const
{
	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_STATUS, NMCS_DISCONNECTED);
}

void CIRCNsm::AddChannel(LPCSTR ch)
{
	std::wstring wch;

	::SJIStoUnicode(ch, wch);

	NSM_SERVICE.CreateGroup(m_hCon, m_gid, wch.c_str());
	NSM_SERVICE.CreateConnectionMember(m_hCon, m_gid, ch, "", NMST_ONLINE);
	m_gid++;
}

void CIRCNsm::RemChannel(int nGroupID) const
{
	NSM_SERVICE.DeleteGroup(m_hCon, nGroupID);
	NSM_SERVICE.ClearGroupMember(m_hCon, nGroupID);
}

int CIRCNsm::GetGroupId(LPCSTR name) const
{
	CCommonService::group_list glist;
	CCommonService::group_list::iterator i;
	wchar_t buf[IRC_MESSAGE_SIZE];
	std::wstring wname;

	::SJIStoUnicode(name, wname);

	NSM_SERVICE.GetConnectionGroupList(glist, m_hCon);
	for( i = glist.begin(); i != glist.end(); ++i ){
		NSM_SERVICE.GetGroupName(m_hCon, *i, buf, sizeof(buf));
		if( wname == buf )
			return *i;
	}

	return 0;
}

void CIRCNsm::ResetChannel(int nGroupID) const
{
	CCommonService::member_list mlist;
	CCommonService::member_list::iterator mi;
	int gid;

	NSM_SERVICE.GetConnectionMemberList(mlist, m_hCon);

	for( mi = mlist.begin(); mi != mlist.end(); ++mi ){
		gid = NSM_SERVICE.GetConnectionMemberInfo(m_hCon, mi->c_str(), NMMI_GROUPID);
		if( (gid == nGroupID) && (mi->find(' ') != mi->npos) )
			NSM_SERVICE.DeleteConnectionMember(m_hCon, mi->c_str());
	}
}

void CIRCNsm::AddMember(LPCSTR ch, LPCSTR nick) const
{
	int gid = GetGroupId(ch);

	if( gid > 0 ){
		std::string acc;
		std::wstring wnick;

		acc = ch;
		acc += " ";
		acc += nick;
		::SJIStoUnicode(nick, wnick);
		NSM_SERVICE.CreateConnectionMember(m_hCon, gid, acc.c_str(), wnick.c_str(), NMST_ONLINE);
	}
}

void CIRCNsm::RemMember(LPCSTR ch, LPCSTR nick) const
{
	std::string acc;

	acc = ch;
	acc += " ";
	acc += nick;
	NSM_SERVICE.DeleteConnectionMember(m_hCon, acc.c_str());
}

void CIRCNsm::ChangeUserName(LPCSTR newName) const
{
	std::wstring wnick;

	::SJIStoUnicode(newName, wnick);
	NSM_SERVICE.SetUserName(m_hCon, wnick.c_str());
//	NSM_SERVICE.SetConnectionInfo(m_hCon, NMCI_CAPTION, wnick.c_str());
}


//******************************************************************
// 
//******************************************************************

CIRCSession* CIRCNsm::CreateSession(LPCSTR name) const
{
	return new CIRCSession(m_hCon, name);
}

void CIRCNsm::GetSessionCaption(HNsmSession hSession, std::string &cap) const
{
	wchar_t buf[IRC_MESSAGE_SIZE];
	int len;

	len = NSM_SERVICE.GetSessionCaption(hSession, buf, sizeof(buf));
	buf[len] = '\0';

	::UnicodeToSJIS(buf, cap);
}

//******************************************************************
// 
//******************************************************************

void CIRCNsm::DoLog(LPCSTR header, LPCSTR body) const
{
	TNsmComLogInfo ncli;
	std::wstring uni;
	std::string utf8;

	::SJIStoUnicode(body, uni);
	::UnicodeToUTF8(uni.c_str(), utf8);

	ncli.cbSize = sizeof(ncli);
	ncli.nType = NMHT_CONNECTION;
	ncli.lpHandle = m_hCon;
	ncli.lpHeader = header;
	ncli.lpMessage = utf8.c_str();
	NSM_SERVICE.NotifyEvent(ms_hLogEvt, (WPARAM)&ncli);
}

void CIRCNsm::Error(LPCSTR msg) const
{
	std::ostringstream oss;

	oss << "G[" << CRLF_STRING;
	oss << msg << CRLF_STRING;
	oss << "G[R[hF" << ::GetLastError() << "/" << ::WSAGetLastError();
	NSM_SERVICE.DefaultNotify(oss.str().c_str(), NMNI_STOP);
}

void CIRCNsm::Warning(LPCSTR msg) const
{
	std::ostringstream oss;

	oss << "x" << CRLF_STRING;
	oss << msg;
	NSM_SERVICE.DefaultNotify(oss.str().c_str(), NMNI_EXCLAMATION);
}

//******************************************************************
// 
//******************************************************************

void CIRCNsm::GetGroupName(int nGroupID, std::string &name) const
{
	wchar_t buf[IRC_MESSAGE_SIZE];
	int len;

	len = NSM_SERVICE.GetGroupName(m_hCon, nGroupID, buf, sizeof(buf));
	buf[len] = '\0';

	::UnicodeToSJIS(buf, name);
}

