#include "StdAfx.h"
#include ".\ircplugin.h"
#include "common.h"
#include "IRCSettings.h"
#include "IRCNsm.h"
#include "../nsmsgs/CommonService.h"
#include "../nsmsgs/mleak.h"

#define EXEC_IRC_CON(hCon, func) \
	{ \
		ConMap::iterator conmap_iterator; \
		conmap_iterator = m_cons.find(hCon); \
		if( conmap_iterator != m_cons.end() ){ \
			conmap_iterator->second->func; \
		} \
	}

CIRCPlugin CIRCPlugin::m_singleton;

//******************************************************************
// RXgN^EfXgN^
//******************************************************************

CIRCPlugin::CIRCPlugin(void)
{
	m_pApiVer		= NSM_API_VERSION;
	m_pModuleName	= NMM_PROTOCOL "/IRC";
	m_pTitle		= "IRC";
	m_pDescription	= "IRC Protocol Module";
	m_pAuthor		= "fors";
	m_pCopyright	= "http://cc.ii2.cc/~fors/sysworks/prog/regne/";
	m_pPluginVer	= "0.0.1x";

	WSADATA wsaData;
	int err;

	m_bWSAStartupSuccess = false;
	err = WSAStartup(MAKEWORD(2,0), &wsaData);
	if( (err == 0 ) && (LOBYTE(wsaData.wVersion) == 2)
		&& (HIBYTE(wsaData.wVersion) == 0) )
	{
		m_bWSAStartupSuccess = true;
	}
}

CIRCPlugin::~CIRCPlugin(void)
{
	if( m_bWSAStartupSuccess )
		WSACleanup();
}

//******************************************************************
// W
//******************************************************************
int CIRCPlugin::OnInitialize()
{
	HNsmEvent evt;

	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_CONNECT, ServiceConConnect);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_DISCONNECT, ServiceConDisconnect);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_CHANGEUSERSTATUS, ServiceConChangeUserStatus);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_CHANGEUSERNAME, ServiceConChangeUserName);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_OPENMAIL, ServiceConOpenMail);

//	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_ADDMEMBER, ServiceConAddMember);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_REMOVEMEMBER, ServiceConRemoveMember);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_CHANGEMEMBERGROUP, ServiceConChangeMemberGroup);

	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_ADDGROUP, ServiceConAddGroup);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_REMOVEGROUP, ServiceConRemoveGroup);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_CONNECTION_CHANGEGROUPNAME, ServiceConChangeGroupName);

	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_OPEN, ServiceSesOpen);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_CLOSE, ServiceSesClose);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_INVITEMEMBER, ServiceSesInviteMember);
	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_SENDMESSAGE, ServiceSesSendMessage);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_BEGINTYPING, ServiceSesBeginTyping);
//	NSM_SERVICE.CreateService(IRC_PROTOCOL_SESSION_ENDTYPING, ServiceSesEndTyping);
  
//	NSM_SERVICE.CreateService(NMS_PROTOCOL_FILESESSION_OPEN, ServiceFSOpen);
//	NSM_SERVICE.CreateService(NMS_PROTOCOL_FILESESSION_ACCEPT, ServiceFSClose);
//	NSM_SERVICE.CreateService(NMS_PROTOCOL_FILESESSION_CANCEL, ServiceFSAccept);
//	NSM_SERVICE.CreateService(NMS_PROTOCOL_FILESESSION_CLOSE, ServiceFSCancel);

	NSM_SERVICE.CreateService(IRC_PROTOCOL_SHOW_OPTION_DIALOG, ServiceShowOptionDialog);

	evt = NSM_SERVICE.CreateEvent(IRC_COM_LOG);
	CIRCNsm::SetLogEvt(evt);

	return NMP_ALLRIGHT;
}

void CIRCPlugin::OnTerminate()
{
}

int CIRCPlugin::OnSystemModulesLoaded()
{
	return 0;
}

int CIRCPlugin::DoShowOptionDialog(HWND hWnd)
{
	IRC_SETTINGS.ShowOptionDialog(GetModuleInstance(), hWnd);
	return 0;
}

//******************************************************************
// T[rX
//******************************************************************

// Connection

int CIRCPlugin::DoConConnect(PLoginInfo pli)
{
	HNsmConnection hCon;
	CIRCConnection *pCon;

	pCon = new CIRCConnection(pli, hCon);
	m_cons[hCon] = pCon;

	return hCon;
}

int CIRCPlugin::DoConDisconnect(HNsmConnection hConnection)
{
	ConMap::iterator conmap_iterator;

	conmap_iterator = m_cons.find(hConnection);
	if( conmap_iterator != m_cons.end() ){
		delete conmap_iterator->second;
_RPT0(_CRT_WARN, "deleted\n");
		m_cons.erase(conmap_iterator);
	}

	return 0;
}
/*
int CIRCPlugin::DoConChangeUserStatus(HNsmConnection hConnection, PUserStatusInfo pusi)
{
	return 0;
}
*/
int CIRCPlugin::DoConChangeUserName(HNsmConnection hConnection, PUserNameInfo puni)
{
	EXEC_IRC_CON(hConnection, ChangeUserName(puni->lpName));
	return 0;
}

/*
int CIRCPlugin::DoConOpenMail(HNsmConnection hConnection)
{
	return 0;
}
*/
// Connection Member
/*
int CIRCPlugin::DoConAddMember(HNsmConnection hConnection, PAddMemberInfo pami)
{
	if( pami->nListKind == NMLK_FORWARDLIST ){
		EXEC_IRC_CON(hConnection, Invite(pami->lpAccount));
	}
	return 0;
}
*/
int CIRCPlugin::DoConRemoveMember(HNsmConnection hConnection, PRemoveMemberInfo prmi)
{
	if( prmi->nListKind == NMLK_FORWARDLIST ){
		EXEC_IRC_CON(hConnection, RemMember(prmi->lpAccount));
	}
	return 0;
}
/*
int CIRCPlugin::DoConChangeMemberGroup(HNsmConnection hConnection, PMemberGroupInfo pmgi)
{
	return 0;
}
*/
// Connection Group

int CIRCPlugin::DoConAddGroup(HNsmConnection hConnection, LPCWSTR lpwGroupName)
{
	EXEC_IRC_CON(hConnection, AddGroup(lpwGroupName));
	return 0;
}

int CIRCPlugin::DoConRemoveGroup(HNsmConnection hConnection, int nGroupID)
{
	EXEC_IRC_CON(hConnection, RemGroup(nGroupID));
	return 0;
}
/*
int CIRCPlugin::DoConChangeGroupName(HNsmConnection hConnection, PGroupNameInfo pgni)
{
	return 0;
}
*/
// Session

int CIRCPlugin::DoSesOpen(HNsmConnection hConnection, POpenSessionInfo posi)
{
	ConMap::iterator conmap_iterator;

	conmap_iterator = m_cons.find(hConnection);
	if( conmap_iterator != m_cons.end() ){
		return conmap_iterator->second->SesOpen(posi->lpToAccount);
	}

	return 0;
}

int CIRCPlugin::DoSesClose(HNsmSession hSession)
{
	HNsmConnection hConnection = NSM_SERVICE.GetConnectionFromSession(hSession);
	EXEC_IRC_CON(hConnection, SesClose(hSession));
	return 0;
}

int CIRCPlugin::DoSesInviteMember(HNsmSession hSession, LPCSTR lpAccount)
{
	HNsmConnection hConnection = NSM_SERVICE.GetConnectionFromSession(hSession);
	EXEC_IRC_CON(hConnection, SesInvite(hSession, lpAccount));
	return 0;
}

int CIRCPlugin::DoSesSendMessage(HNsmSession hSession, PMessageInfo pmi)
{
	HNsmConnection hConnection = NSM_SERVICE.GetConnectionFromSession(hSession);
	EXEC_IRC_CON(hConnection, SesSendMsg(hSession, pmi->lpBody));
	return 0;
}
/*
int CIRCPlugin::DoSesBeginTyping(HNsmSession hSession)
{
	return 0;
}

int CIRCPlugin::DoSesEndTyping(HNsmSession hSession)
{
	return 0;
}

// File Session
int CIRCPlugin::DoFSOpen(HNsmSession hSession, POpenFileSessionInfo pofsi)
{
	return 0;
}

int CIRCPlugin::DoFSClose(HNsmFileSession hFileSession)
{
	return 0;
}

int CIRCPlugin::DoFSAccept(HNsmFileSession hFileSession)
{
	return 0;
}

int CIRCPlugin::DoFSCancel(HNsmFileSession hFileSession)
{
	return 0;
}
*/
