#include "stdafx.h"
#include "IRCSession.h"
#include "../nsmsgs/CommonService.h"
#include "../nsmsgs/cs_conv.h"
#include "../nsmsgs/DynamicArray.h"

//******************************************************************
// 
//******************************************************************

CIRCSession::CIRCSession(HNsmConnection hCon, LPCSTR name)
: m_name(name)
, m_hSes(NSM_SERVICE.CreateSession(hCon))
, m_user_id(0)
, m_bConnected(false)
{
	std::wstring wch;

	::SJIStoUnicode(name, wch);
	NSM_SERVICE.SetSessionCaption(m_hSes, wch.c_str());
	NSM_SERVICE.SetSessionState(m_hSes, NMCS_CONNECTING);
}

CIRCSession::~CIRCSession()
{
	Users::iterator i;

	NSM_SERVICE.DeleteSession(m_hSes);

	for( i = m_users.begin(); i != m_users.end(); ++i )
		delete i->second;
}

//******************************************************************
// 
//******************************************************************

void CIRCSession::Connected()
{
	m_bConnected = true;
	NSM_SERVICE.SetSessionState(m_hSes, NMCS_CONNECTED);
}

void CIRCSession::Disconnect() const
{
	NSM_SERVICE.SetSessionState(m_hSes, NMCS_DISCONNECTED);
}

void CIRCSession::AddMember(LPCSTR nick, LPCSTR acc, u_int mode)
{
	CIRCUser *u;
	std::wstring wnick;
	MemberStatus status;
	const int bufSize = strlen(acc)+16;
	DynamicArray<char> buf(bufSize);

	if( m_users.find(acc) != m_users.end() ){
		_ASSERT(FALSE);
		return;
	}

	m_user_id++;
	_itoa_s(m_user_id, buf, 10, 10);
	strcat_s(buf, bufSize, "!");
	strcat_s(buf, bufSize, acc);

	u = new CIRCUser(nick, buf, mode);
	m_users[u->GetMapKey()] = u;

	::SJIStoUnicode(nick, wnick);
	status = (mode & CIRCUser::MODE_CHOP) ? NMST_BUSY: NMST_ONLINE;
	NSM_SERVICE.CreateSessionMember(m_hSes, buf);
	NSM_SERVICE.SetSessionMemberInfo(m_hSes, buf, NMMI_STATUS, status);
	NSM_SERVICE.SetSessionMemberInfo(m_hSes, buf, NMMI_NAME, wnick.c_str());
}

void CIRCSession::RemMember(LPCSTR nick)
{
	Users::iterator i;
	CIRCUser *u;

	i = m_users.find(nick);
	if( i == m_users.end() ){
		_ASSERT(FALSE);
		return;
	}

	u = i->second;
	NSM_SERVICE.DeleteSessionMember(m_hSes, u->GetAccount());
	m_users.erase(i);
	delete u;
}

void CIRCSession::SetMemberName(LPCSTR oldName, LPCSTR newName)
{
	std::wstring wmember;
	Users::iterator i;
	CIRCUser *u;
	
	i = m_users.find(oldName);
	if( i == m_users.end() ){
		_ASSERT(FALSE);
		return;
	}

	u = i->second;
	m_users.erase(i);

	u->SetNick(newName);
	m_users[u->GetMapKey()] = u;

	::SJIStoUnicode(newName, wmember);
	NSM_SERVICE.SetSessionMemberInfo(
		m_hSes, u->GetAccount(), NMMI_NAME, wmember.c_str());
}

void CIRCSession::SetMemberMode(LPCSTR nick, u_int mode)
{
	std::wstring wmember;
	Users::iterator i;
	CIRCUser *u;
	u_int status;
	
	i = m_users.find(nick);
	if( i == m_users.end() ){
		_ASSERT(FALSE);
		return;
	}
	u = i->second;
	u->SetMode(mode);

	status = (mode & CIRCUser::MODE_CHOP) ? NMST_BUSY: NMST_ONLINE;
	NSM_SERVICE.SetSessionMemberInfo(m_hSes, u->GetAccount(), NMMI_STATUS, status);
}

void CIRCSession::SetTopic(LPCSTR topic) const
{
	std::string cap;
	std::wstring wcap;

	cap = m_name;
	if( *topic != '\0' ){
		cap += ": ";
		cap += topic;
	}
	::SJIStoUnicode(cap.c_str(), wcap);
	NSM_SERVICE.SetSessionCaption(m_hSes, wcap.c_str());
}

void CIRCSession::RecvMessage(LPCSTR from, LPCSTR msg) const
{
	Users::const_iterator i;
	std::wstring wmsg;
	TMessageInfo mi;
	TTextAttributeInfo tai;

	i = m_users.find(from);
	if( i == m_users.end() ){
		_ASSERT(FALSE);
		return;
	}

	::SJIStoUnicode(msg, wmsg);

	mi.cbSize = sizeof(mi);
	mi.lpFrom = i->second->GetAccount();
	mi.lpBody = wmsg.c_str();
	mi.lpTextAttribute = &tai;
	mi.nFlags = NULL;

	tai.cbSize = sizeof(tai);
	tai.lpFontName = NULL;
	tai.nCharSet = SHIFTJIS_CHARSET;
	tai.nFontSize = -1;
	tai.nFontColor = -1;
	tai.nBgColor = -1;
	tai.nStyles = NULL;

	NSM_SERVICE.RecvMessage(m_hSes, &mi);
}
