#pragma once
#include <map>
#include <string>
#include <functional>
#include "IRCUser.h"

struct spless
	: public std::binary_function<LPCSTR, LPCSTR, bool>
{
	bool operator ()(const LPCSTR &x, const LPCSTR &y) const
	{ return strcmp(x, y) < 0; }
};

class CIRCSession
{
private:
	typedef std::map<LPCSTR, CIRCUser*, spless> Users;

private:
	std::string m_name;
	HNsmSession m_hSes;
	Users m_users;
	u_int m_user_id;
	bool m_bConnected;

public:
	CIRCSession(HNsmConnection hCon, LPCSTR name);
	~CIRCSession();

	void Connected();
	void Disconnect() const;
	void AddMember(LPCSTR nick, LPCSTR acc, u_int mode=0);
	void RemMember(LPCSTR nick);
	void SetMemberName(LPCSTR oldName, LPCSTR newName);
	void SetMemberMode(LPCSTR nick, u_int mode);
	void SetTopic(LPCSTR topic) const;
	void RecvMessage(LPCSTR from, LPCSTR msg) const;

	LPCSTR GetName() const
	{ return m_name.c_str(); }

	HNsmSession GetNsmHandle() const
	{ return m_hSes; }

	bool IsConnected() const
	{ return m_bConnected; }
};