#include "StdAfx.h"
#include ".\ircsettings.h"
#include <sstream>
#include "common.h"
#include "AccPage.h"
#include "CmnPage.h"
#include "../nsmsgs/PSheet.h"
#include "../nsmsgs/general_strings.h"

const static char SETTING_DLG_NAME[]	= "IRCݒ";
const static char SN_ACCOUNTS[]			= "Accounts";
const static char KN_ACCOUNT[]			= "Account";
const static char KN_HOST[]				= "Host";
const static char KN_PORT[]				= "Port";
const static char KN_PASS[]				= "Pass";
const static char KN_NICK[]				= "Nick";
const static char KN_CHANNEL[]			= "Channel";

CIRCSettings CIRCSettings::m_singleton(PLUGIN_NAME);

//******************************************************************
// 
//******************************************************************

CIRCSettings::CIRCSettings(LPCTSTR pluginName)
: COptionSettings(pluginName)
{
}

CIRCSettings::~CIRCSettings(void)
{
}

//******************************************************************
// 
//******************************************************************

bool CIRCSettings::GetAccSetting(AccSetting &set)
{
	int no;
	char buf[16];
	std::string ch;
	LPCSTR ch_st, ch_ed;
	ConChannel *cc;
	int len;

	no = GetAccountNo(set.acc.c_str());
	if( no <= 0 ) return false;

	_itoa_s(no, buf, 10);
	m_ini.ReadString(set.host, buf, KN_HOST, NULL_STRING);
	set.port = (unsigned short)m_ini.ReadInt(buf, KN_PORT, 0);
	m_ini.ReadString(set.pass, buf, KN_PASS, NULL_STRING);
	m_ini.ReadString(set.nick, buf, KN_NICK, NULL_STRING);
	m_ini.ReadString(ch, buf, KN_CHANNEL, NULL_STRING);

	ch_st = ch.c_str();
	for(;;){
		ch_ed = strchr(ch_st, ' ');
		if( ch_ed == NULL ) break;
		if( ch_ed - ch_st < 2 ) break;
		cc = new ConChannel;
		cc->bAutoJoin = (*ch_st == '1');
		ch_st++;
		cc->name.assign(ch_st, ch_ed - ch_st);
		set.chs.push_back(cc);
		ch_st = ch_ed + 1;
	}
	len = strlen(ch_st);
	if( len > 0 ){
		cc = new ConChannel;
		cc->bAutoJoin = (*ch_st == '1');
		cc->name.assign(ch_st+1, len-1);
		set.chs.push_back(cc);
	}

	return true;
}

void CIRCSettings::SetConChList(LPCSTR acc, const ConChList &ccl)
{
	char buf[16];
	std::ostringstream oss;
	int ed;
	int no;

	no = GetAccountNo(acc);
	if( no <= 0 ) return;

	ed = ccl.size();
	for( int i = 0; i < ed; ++i ){
		ConChannel *cc = ccl[i];
		oss << cc->bAutoJoin << cc->name << SPACE_DELIMITER;
	}

	_itoa_s(no, buf, 10);
	m_ini.Write(buf, KN_CHANNEL, oss.str().c_str());
}

void CIRCSettings::ShowOptionDialog(HINSTANCE hInst, HWND hWnd)
{
	AccountList al;
	CommonSettings cs;

	GetAllAccountInfo(al);
	GetCommonSettings(cs);

	CAccPage ap(al);
	CCmnPage cp(cs);
	CPSheet ps;

	ps.Add(ap);
//	ps.Add(cp);

	if( ps.DoModal(hInst, hWnd, SETTING_DLG_NAME) ){
		SetCommonSettings(cs);
		SetAllAccountInfo(al);
	}
	UnloadAllAccountInfo(al);
}

//******************************************************************
// 
//******************************************************************

int CIRCSettings::GetAccountNo(LPCSTR acc)
{
	return m_ini.ReadInt(SN_ACCOUNTS, acc, 0);
}

void CIRCSettings::GetAllAccountInfo(AccountList &list)
{
	CIniFile::EntryList accounts;
	CIniFile::EntryList::iterator i;
	std::string acc;
	AccSetting *as;
	char buf[16];

	for( int no = 1; ; no++ ){
		_itoa_s(no, buf, 10);
		m_ini.ReadString(acc, buf, KN_ACCOUNT, NULL_STRING);
		if( acc.length() == 0 ) break;

		as = new AccSetting;
		as->acc = acc;
		GetAccSetting(*as);
		list.push_back(as);
	}
}

void CIRCSettings::GetCommonSettings(CommonSettings &/*cs*/)
{
}

void CIRCSettings::SetAllAccountInfo(AccountList &list)
{
	AccountList::iterator li;
	AccSetting *as;
	int size;

	// ̃ZNVj
	m_ini.DeleteSection(SN_ACCOUNTS);
	DeleteAllAccountSections();

	// o
	size = list.size();
	for( int i = 0; i < size; ++i ){
		as = list[i];
		SetAccSetting(i+1, *as);
		m_ini.Write(SN_ACCOUNTS, as->acc.c_str(), i+1);
	}
}

void CIRCSettings::SetCommonSettings(CommonSettings &/*cs*/)
{
}

void CIRCSettings::UnloadAllAccountInfo(AccountList &list)
{
	int asize;

	asize = list.size();
	for( int i = 0; i < asize; ++i ){
		AccSetting *as = list[i];
		as->DeleteConChList();
		delete as;
	}
}

void CIRCSettings::DeleteAllAccountSections()
{
	CIniFile::SectionList sections;
	CIniFile::SectionList::iterator si;

	m_ini.GetSectionList(sections);
	for( si = sections.begin(); si != sections.end(); ++si ){
		if( atoi(si->c_str()) != 0 ){
			m_ini.DeleteSection(si->c_str());
		}
	}
}

void CIRCSettings::SetAccSetting(int no, AccSetting &set)
{
	char buf[16];
	std::ostringstream oss;
	int ed;

	ed = set.chs.size();
	for( int i = 0; i < ed; ++i ){
		ConChannel *cc = set.chs[i];
		oss << cc->bAutoJoin << cc->name << SPACE_DELIMITER;
	}

	_itoa_s(no, buf, 10);
	m_ini.Write(buf, KN_ACCOUNT, set.acc.c_str());
	m_ini.Write(buf, KN_HOST, set.host.c_str());
	m_ini.Write(buf, KN_PORT, set.port);
	m_ini.Write(buf, KN_PASS, set.pass.c_str());
	m_ini.Write(buf, KN_NICK, set.nick.c_str());
	m_ini.Write(buf, KN_CHANNEL, oss.str().c_str());
}

//******************************************************************
// 
//******************************************************************

