#include "StdAfx.h"
#include <process.h>
#include ".\ircsocket.h"
#include "../nsmsgs/cs_conv.h"
#include "../nsmsgs/general_strings.h"
#include "../nsmsgs/CommonService.h"

#define WM_USER_ASYNC_GHBN	(WM_USER + 1)

#define SOCK_TIMEOUT	10000

const static char ERR_SERVER_NOT_FOUND[]	= "T[o܂";
const static char ERR_TIMEOUT_RESOLVE[]		= "T[ỏɃ^CAEg܂";
const static char ERR_CANT_CREATE_SOCKET[]	= "\Pbg̍쐬Ɏs܂";
const static char ERR_CANT_CONNECT_SERVER[]	= "T[oɐڑł܂";
const static char ERR_CANT_SEND[]			= "MɎs܂";
const static char ERR_CON_TIMEOUT[]			= "ڑɃ^CAEg܂";

const static char ERR_WAIT_FAILED[]			= "Cxg҂Ɏs܂";
const static char ERR_ENUM_EVENTS[]			= "Cxg񋓂Ɏs܂";
const static char ERR_EVENT_SELECT[]		= "CxgZbgɎs܂";
const static char ERR_DISCONNECTED[]		= "T[oɐؒf܂";

const static char ERR_ASYNC_GHBN_FUNC[]		= "zXg̉ɓG[܂";
const static char ERR_ASYNC_GHBN_MSG[]		= "zXg̉Ɏs܂";
const static char ERR_GHBN_TIMEOUT[]		= "zXg̉Ƀ^CAEg܂";

const static char ERR_RECV[]				= "MɎs܂";
const static char ERR_CHECK_RECV[]			= "M҂Ɏs܂";
const static char ERR_LINE_EXCEED[]			= "Mf[^1s512oCg𒴂܂";

const static char ERR_SET_WAITABLE_TIMER[]	= "^C}ZbgɎs܂";

//******************************************************************
// 
//******************************************************************

CIRCSocket::CIRCSocket(LPCSTR host, u_short port, COut &out)
: m_socket(INVALID_SOCKET)
, m_buf_filled(0)
, m_host(host)
, m_port(port)
, m_out(&out)
, m_maxPenalty(10)
{
	::InitializeCriticalSection(&m_SendCS);

	::GetSystemTimeAsFileTime(m_tbase);
	m_hTimer = ::CreateWaitableTimer(NULL, TRUE, NULL);

	m_hEvents[EVENT_STOP] = ::CreateEvent(NULL, TRUE, FALSE, NULL);
	m_hEvents[EVENT_SOCKET] = ::CreateEvent(NULL, FALSE, FALSE, NULL);
	m_hEvents[EVENT_QUEUE] = ::CreateEvent(NULL, FALSE, FALSE, NULL);
	_ASSERT( m_hEvents[EVENT_STOP] != INVALID_HANDLE_VALUE );
	_ASSERT( m_hEvents[EVENT_SOCKET] != INVALID_HANDLE_VALUE );
	_ASSERT( m_hEvents[EVENT_QUEUE] != INVALID_HANDLE_VALUE );

	m_hThread = (HANDLE)_beginthreadex(NULL, 0, ServerThread, this, 0, &m_threadID);
}

CIRCSocket::~CIRCSocket(void)
{
	SetEvent(m_hEvents[EVENT_STOP]);
	while( WaitForSingleObjectEx(m_hThread, INFINITE, TRUE) != WAIT_OBJECT_0 );
	CloseHandle(m_hThread);

	while( m_sq.size() > 0 ){
		delete m_sq.front();
		m_sq.pop();
	}
	::CloseHandle(m_hEvents[EVENT_QUEUE]);
	::CloseHandle(m_hEvents[EVENT_SOCKET]);
	::CloseHandle(m_hEvents[EVENT_STOP]);

	::CloseHandle(m_hTimer);

	::DeleteCriticalSection(&m_SendCS);
}

//******************************************************************
// OXbhp
//******************************************************************

void CIRCSocket::Send(LPCSTR line, int penalty)
{
	SendQueueEntry *e = new SendQueueEntry;

	e->msg = line;
	e->penalty = penalty;

	::EnterCriticalSection(&m_SendCS);
	m_sq.push(e);
	if( m_sq.size() == 1 )
		::SetEvent(m_hEvents[EVENT_QUEUE]);
	::LeaveCriticalSection(&m_SendCS);
}

//******************************************************************
// Xbhp
//******************************************************************

void CIRCSocket::ThreadBody()
{
	try{
		Connect();
		SocketLoop();
	}catch(LPCSTR msg){
		_RPT1(_CRT_WARN, "IRCSocket Error: %s\n", msg);
		if( *msg != '\0' ){
			DoError(msg);
		}else{
			Quit();
		}
	}

	if( m_socket != INVALID_SOCKET )
		Disconnect();
}

void CIRCSocket::Connect()
{
	_ASSERT( m_socket == INVALID_SOCKET );
	struct sockaddr_in addr;
	struct in_addr host;
	SOCKET s;
	LPCSTR err;
	int ret;

	// ڑ擾
	if( (m_host.length() > 0) && isdigit(m_host[m_host.length()-1]) ){
		host.S_un.S_addr = inet_addr(m_host.c_str());
	}else{
		host = GetHostByName(m_host.c_str());
		if( host.S_un.S_addr == NULL )
			throw ERR_SERVER_NOT_FOUND;
	}

	// ʐMO
	{
		char buf[40];
		_snprintf_s(buf, sizeof(buf), "Connecting to %d.%d.%d.%d:%d",
			host.S_un.S_un_b.s_b1, host.S_un.S_un_b.s_b2,
			host.S_un.S_un_b.s_b3, host.S_un.S_un_b.s_b4,
			m_port
		);
		DoLog("-", buf);
	}

	addr.sin_family = PF_INET;
	addr.sin_addr = host;
	addr.sin_port = htons(m_port);

	// \Pbg쐬
	s = socket(PF_INET, SOCK_STREAM, 0);
	if( s == INVALID_SOCKET ){
		throw ERR_CANT_CREATE_SOCKET;
	}
	ret = FALSE;
	ret = WSAEventSelect(s, m_hEvents[EVENT_SOCKET], FD_READ|FD_CONNECT|FD_CLOSE);
	if( ret == SOCKET_ERROR ) throw ERR_EVENT_SELECT;

	// ڑ
	ret = connect(s, (sockaddr*)&addr, sizeof(addr));
	if( (ret != 0) && (WSAGetLastError() != WSAEWOULDBLOCK) ){
		closesocket(s);
		throw ERR_CANT_CONNECT_SERVER;
	}
	m_socket = s;

	err = WaitConnect();
	if( err != NULL ) throw err;
}

// ؒf
void CIRCSocket::Disconnect()
{
	char buf[256];

	shutdown(m_socket, SD_BOTH);
	while( recv(m_socket, buf, sizeof(buf), 0) > 0 );
	closesocket(m_socket);
	m_socket = INVALID_SOCKET;
}

void CIRCSocket::SocketLoop()
{
	DWORD ret;

	for(;;){
		ret = ::WaitForMultipleObjectsEx(NUM_OF_EVENTS, m_hEvents, FALSE, INFINITE, TRUE);

		if( ret == WAIT_OBJECT_0 + EVENT_QUEUE ){
			BeginSend();
		}else if( ret == WAIT_OBJECT_0 + EVENT_SOCKET ){
			ret = CheckWSAEvt(FD_READ_BIT);
			if( ret == -1 ) continue;
			if( ret != 0 ) throw ERR_CHECK_RECV;
			Recv();
		}else if( ret != WAIT_IO_COMPLETION ){
			throw NULL_STRING;
		}
	}
}

void CIRCSocket::Quit()
{
	std::string msg;

	msg = "QUIT :";
	msg += "QUIT";
	msg += CRLF_STRING;
	try{
		Send(msg);
	}catch(...){}
}

CIRCSocket::SendQueueEntry* CIRCSocket::PopSendQueue()
{
	SendQueueEntry *e = NULL;

	::EnterCriticalSection(&m_SendCS);
	if( m_sq.size() > 0 ){
		e = m_sq.front();
		m_sq.pop();
	}else{
		::ResetEvent(m_hEvents[EVENT_QUEUE]);
	}
	::LeaveCriticalSection(&m_SendCS);

	return e;
}

CIRCSocket::SendQueueEntry* CIRCSocket::PeekSendQueue()
{
	SendQueueEntry *e = NULL;

	::EnterCriticalSection(&m_SendCS);
	if( m_sq.size() > 0 ) e = m_sq.front();
	::LeaveCriticalSection(&m_SendCS);

	return e;
}

void CIRCSocket::BeginSend()
{
	SendQueueEntry *e = PeekSendQueue();

	_ASSERT( e != NULL );
	if( e == NULL ) return;
	SetSendTimer(e->penalty);
}

void CIRCSocket::SendFromQueue()
{
	SendQueueEntry *e = NULL;
	SendQueueEntry *next = NULL;

	::EnterCriticalSection(&m_SendCS);
	if( m_sq.size() >= 1 ){ // pop first
		e = m_sq.front();
		m_sq.pop();
	}
	if( m_sq.size() >= 1 ){	// peek next first
		next = m_sq.front();
	}
	if( m_sq.size() == 0 ){
		::ResetEvent(m_hEvents[EVENT_QUEUE]);
	}
	::LeaveCriticalSection(&m_SendCS);

	_ASSERT( e != NULL );
	if( e == NULL ) return;
	Send(e->msg);
	delete e;

	if( next == NULL ) return;
	SetSendTimer(next->penalty);
}

void CIRCSocket::SetSendTimer(int penalty)
{
	FILETIME timeout;
	BOOL ret;

	CalcNextSendTime(timeout, penalty);
	ret = ::SetWaitableTimer(m_hTimer, (LARGE_INTEGER*)&timeout, 0, SendFromQueue, this, FALSE);
	if( ! ret ){
		throw ERR_SET_WAITABLE_TIMER;
	}
}

void CIRCSocket::CalcNextSendTime(FILETIME &timeout, int penalty)
{
	CFileTime now;

	timeout = m_tbase - m_maxPenalty;

	::GetSystemTimeAsFileTime(now);
	if( m_tbase < now ){
		m_tbase = now;
	}
	m_tbase += penalty;
}

void CIRCSocket::Recv()
{
	int buf_remain;
	LPSTR buf_st;
	int len;
	LPSTR st, ed;

	for(;;){
		buf_remain = sizeof(m_buf) - m_buf_filled - 1;
		buf_st = m_buf + m_buf_filled;
		_ASSERT(buf_remain > 0);

		// M
		len = recv(m_socket, buf_st, buf_remain, 0);
		if( len == SOCKET_ERROR ){
			if( ::WSAGetLastError() == WSAEWOULDBLOCK )
				break;
			throw ERR_RECV;
		}
		if( len == 0 ) throw ERR_DISCONNECTED;

		m_buf_filled += len;
		_ASSERT(m_buf_filled < sizeof(m_buf));

		// 1sƂExec֓n
		m_buf[m_buf_filled] = '\0';
		st = m_buf;
		for(;;){
			ed = strstr(st, CRLF_STRING);
			if( ed == NULL ) break;
			*ed = '\0';
			Exec(st);
			st = ed + LEN_OF(CRLF_STRING);
		}

		// cO
		if( m_buf < st ){
			m_buf_filled -= st - m_buf;
			memmove(m_buf, st, m_buf_filled);
		}else if( m_buf_filled >= sizeof(m_buf) - 1 ){
			throw ERR_LINE_EXCEED;
		}
	}
}

int CIRCSocket::CheckWSAEvt(int evt) const
{
	int ret;
	WSANETWORKEVENTS ne;

	ZeroMemory(&ne, sizeof(ne));
	ret = WSAEnumNetworkEvents(m_socket, m_hEvents[EVENT_SOCKET], &ne);
	if( ret == SOCKET_ERROR )
		throw ERR_ENUM_EVENTS;
	if( (ne.lNetworkEvents >> evt) & 0x1 )
		return ne.iErrorCode[evt];
	if( ne.lNetworkEvents & FD_CLOSE )
		throw ERR_DISCONNECTED;
	return -1;
}

void CIRCSocket::Send(std::string &str) const
{
	int ret;
	std::string jis;
	fd_set w_fd;
	timeval timeout;

	DoLog("C", str.c_str());
	SJIStoJIS(str.c_str(), jis);

	FD_ZERO(&w_fd);
	FD_SET(m_socket, &w_fd);
	timeout.tv_sec = 1;
	timeout.tv_usec = 0;
//	if( select(0, NULL, &w_fd, NULL, &timeout) <= 0 ){
//		throw ERR_CANT_SEND;
//	}
	ret = send(m_socket, jis.c_str(), jis.length(), 0);
	if( ret != (int)jis.length() ){
		throw ERR_CANT_SEND;
	}
}

void CIRCSocket::Exec(LPCSTR line)
{
	char buf[IRC_MESSAGE_SIZE];
	std::string utf8;

	JIStoSJIS(buf, line, line+strlen(line));
	DoLog("S", buf);

	m_out->DoExec(buf);
}

// 񓯊҂
LPCSTR CIRCSocket::WaitConnect() const
{
	enum{
		L_EVENT_STOP,
		L_EVENT_SOCKET,
		L_NUM_OF_EVENTS
	};
	HANDLE evts[L_NUM_OF_EVENTS];
	int ret;

	evts[L_EVENT_STOP] = m_hEvents[EVENT_STOP];
	evts[L_EVENT_SOCKET] = m_hEvents[EVENT_SOCKET];

	ret = WSAWaitForMultipleEvents(L_NUM_OF_EVENTS, evts, FALSE, SOCK_TIMEOUT, FALSE);
	if( ret == WSA_WAIT_TIMEOUT ) return ERR_CON_TIMEOUT;
	if( ret == WSA_WAIT_FAILED ) return ERR_WAIT_FAILED;
	if( ret == (WSA_WAIT_EVENT_0 + L_EVENT_STOP) )
		return NULL_STRING;

	ret = CheckWSAEvt(FD_CONNECT_BIT);
	if( ret != 0 ) return ERR_CANT_CONNECT_SERVER;

	return NULL;
}

// 荞݉\zXg
struct in_addr CIRCSocket::GetHostByName(LPCSTR name) const
{
	enum{
		L_EVENT_STOP,
		L_EVENT_THREAD,
		L_NUM_OF_EVENTS
	};
	HANDLE evts[L_NUM_OF_EVENTS];
	int ret;
	struct in_addr value;

	evts[L_EVENT_STOP] = m_hEvents[EVENT_STOP];
	evts[L_EVENT_THREAD] = (HANDLE)_beginthreadex(NULL, 0, AsyncGetHostByName, (LPVOID)name, 0, NULL);
	ret = WaitForMultipleObjects(L_NUM_OF_EVENTS, evts, FALSE, SOCK_TIMEOUT);
	if( ret == WAIT_TIMEOUT ){
		CloseHandle(evts[L_EVENT_THREAD]);
		throw ERR_TIMEOUT_RESOLVE;
	}

	GetExitCodeThread(evts[L_EVENT_THREAD], (LPDWORD)&value);
	CloseHandle(evts[L_EVENT_THREAD]);

	if( ret == (WAIT_OBJECT_0 + L_EVENT_STOP) )
		throw NULL_STRING;

	return value;
}


//******************************************************************
// 
//******************************************************************

// zXgXbh
unsigned int __stdcall CIRCSocket::AsyncGetHostByName(void* param)
{
	struct hostent *pHost = gethostbyname((LPCSTR)param);
	if( (pHost != NULL) && (pHost->h_length > 0) )
		return *(unsigned int*)pHost->h_addr;
	return NULL;
}

unsigned int __stdcall CIRCSocket::ServerThread(void* param)
{
	((CIRCSocket*)param)->ThreadBody();
	return 0;
}

VOID CALLBACK CIRCSocket::SendFromQueue(LPVOID param, DWORD /*nowLow*/, DWORD /*nowHigh*/)
{
	((CIRCSocket*)param)->SendFromQueue();
}


