#pragma once
#include <string>
#include <queue>
#include "FileTime.h"
#include "Out.h"
#include "common.h"

class CIRCSocket
{
private:
	enum{
		EVENT_STOP,
		EVENT_SOCKET,
		EVENT_QUEUE,
		NUM_OF_EVENTS
	};

public:
	struct SendQueueEntry
	{
		std::string msg;
		int penalty;
	};

private:
	typedef std::queue<SendQueueEntry*> SendQueue;
	enum{ FLOOD_LIMIT_TIME=2000 };

private:
	SOCKET m_socket;
	char m_buf[IRC_MESSAGE_SIZE+1];
	int m_buf_filled;
	SendQueue m_sq;

	HANDLE m_hEvents[NUM_OF_EVENTS];

	const std::string m_host;
	const u_short m_port;
	COut * const m_out;

	unsigned int m_threadID;
	HANDLE m_hThread;
	CRITICAL_SECTION m_SendCS;

	HANDLE m_hTimer;
	CFileTime m_tbase;
	int m_maxPenalty;

public:
	CIRCSocket(LPCSTR host, u_short port, COut &out);
	~CIRCSocket(void);
	CIRCSocket& operator =(CIRCSocket& /*other*/)
	{ _ASSERT(FALSE); return *this; }

	void Send(LPCSTR line, int penalty);

private:
	void ThreadBody();
	void Connect();
	void Disconnect();
	void SocketLoop();
	void Quit();

	SendQueueEntry* PopSendQueue();
	SendQueueEntry* PeekSendQueue();
	void BeginSend();
	void SendFromQueue();
	void SetSendTimer(int penalty);
	void CalcNextSendTime(FILETIME &timeout, int penalty);
	void Recv();
	int CheckWSAEvt(int evt) const;

	LPCSTR WaitConnect() const;
	void Send(std::string &str) const;
	void Exec(LPCSTR line);
	struct in_addr GetHostByName(LPCSTR name) const;

	void DoLog(LPCSTR header, LPCSTR body) const
	{ m_out->DoLog(header, body); }
	void DoError(LPCSTR msg) const
	{ m_out->DoError(msg); }

private:
	static unsigned int __stdcall AsyncGetHostByName(void* param);
	static unsigned int __stdcall ServerThread(void* param);
	static VOID CALLBACK SendFromQueue(LPVOID param, DWORD nowLow, DWORD nowHigh);
};
